var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
let ls$1 = {};
class MemoryStorageInterface {
  constructor() {
    Object.defineProperty(this, "length", {
      get() {
        return Object.keys(ls$1).length;
      }
    });
  }
  getItem(name) {
    return name in ls$1 ? ls$1[name] : null;
  }
  setItem(name, value) {
    ls$1[name] = value;
    return true;
  }
  removeItem(name) {
    const found = name in ls$1;
    if (found) {
      return delete ls$1[name];
    }
    return false;
  }
  key(index) {
    const keys = Object.keys(ls$1);
    return typeof keys[index] !== "undefined" ? keys[index] : null;
  }
}
const MemoryStorage = new MemoryStorageInterface();
const listeners = {};
class WebStorageEvent {
  static on(name, callback) {
    if (typeof listeners[name] === "undefined") {
      listeners[name] = [];
    }
    listeners[name].push(callback);
  }
  static off(name, callback) {
    if (listeners[name].length) {
      listeners[name].splice(listeners[name].indexOf(callback), 1);
    } else {
      listeners[name] = [];
    }
  }
  static emit(event) {
    const e = event || window.event;
    const getValue = (data) => {
      try {
        return JSON.parse(data).value;
      } catch (err) {
        return data;
      }
    };
    const fire = (listener) => {
      const newValue = getValue(e.newValue);
      const oldValue = getValue(e.oldValue);
      listener(newValue, oldValue, e.url || e.uri);
    };
    if (typeof e === "undefined" || typeof e.key === "undefined") {
      return;
    }
    const all = listeners[e.key];
    if (typeof all !== "undefined") {
      all.forEach(fire);
    }
  }
}
class WebStorage {
  constructor(storage) {
    this.storage = storage;
    this.options = {
      namespace: "",
      events: ["storage"]
    };
    Object.defineProperty(this, "length", {
      get() {
        return this.storage.length;
      }
    });
    if (typeof window !== "undefined") {
      for (const i in this.options.events) {
        if (window.addEventListener) {
          window.addEventListener(this.options.events[i], WebStorageEvent.emit, false);
        } else if (window.attachEvent) {
          window.attachEvent(`on${this.options.events[i]}`, WebStorageEvent.emit);
        } else {
          window[`on${this.options.events[i]}`] = WebStorageEvent.emit;
        }
      }
    }
  }
  setOptions(options = {}) {
    this.options = Object.assign(this.options, options);
  }
  set(name, value, expire = null) {
    const stringifyValue = JSON.stringify({
      value,
      expire: expire !== null ? new Date().getTime() + expire : null
    });
    this.storage.setItem(this.options.namespace + name, stringifyValue);
  }
  get(name, def = null) {
    const item = this.storage.getItem(this.options.namespace + name);
    if (item !== null) {
      try {
        const data = JSON.parse(item);
        if (data.expire === null) {
          return data.value;
        }
        if (data.expire >= new Date().getTime()) {
          return data.value;
        }
        this.remove(name);
      } catch (err) {
        return def;
      }
    }
    return def;
  }
  key(index) {
    return this.storage.key(index);
  }
  remove(name) {
    return this.storage.removeItem(this.options.namespace + name);
  }
  clear() {
    if (this.length === 0) {
      return;
    }
    const removedKeys = [];
    for (let i = 0; i < this.length; i++) {
      const key = this.storage.key(i);
      const regexp = new RegExp(`^${this.options.namespace}.+`, "i");
      if (regexp.test(key) === false) {
        continue;
      }
      removedKeys.push(key);
    }
    for (const key in removedKeys) {
      this.storage.removeItem(removedKeys[key]);
    }
  }
  on(name, callback) {
    WebStorageEvent.on(this.options.namespace + name, callback);
  }
  off(name, callback) {
    WebStorageEvent.off(this.options.namespace + name, callback);
  }
}
/*! js-cookie v3.0.1 | MIT */
function assign(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];
    for (var key in source) {
      target[key] = source[key];
    }
  }
  return target;
}
var defaultConverter = {
  read: function(value) {
    if (value[0] === '"') {
      value = value.slice(1, -1);
    }
    return value.replace(/(%[\dA-F]{2})+/gi, decodeURIComponent);
  },
  write: function(value) {
    return encodeURIComponent(value).replace(/%(2[346BF]|3[AC-F]|40|5[BDE]|60|7[BCD])/g, decodeURIComponent);
  }
};
function init(converter, defaultAttributes) {
  function set(key, value, attributes) {
    if (typeof document === "undefined") {
      return;
    }
    attributes = assign({}, defaultAttributes, attributes);
    if (typeof attributes.expires === "number") {
      attributes.expires = new Date(Date.now() + attributes.expires * 864e5);
    }
    if (attributes.expires) {
      attributes.expires = attributes.expires.toUTCString();
    }
    key = encodeURIComponent(key).replace(/%(2[346B]|5E|60|7C)/g, decodeURIComponent).replace(/[()]/g, escape);
    var stringifiedAttributes = "";
    for (var attributeName in attributes) {
      if (!attributes[attributeName]) {
        continue;
      }
      stringifiedAttributes += "; " + attributeName;
      if (attributes[attributeName] === true) {
        continue;
      }
      stringifiedAttributes += "=" + attributes[attributeName].split(";")[0];
    }
    return document.cookie = key + "=" + converter.write(value, key) + stringifiedAttributes;
  }
  function get(key) {
    if (typeof document === "undefined" || arguments.length && !key) {
      return;
    }
    var cookies = document.cookie ? document.cookie.split("; ") : [];
    var jar = {};
    for (var i = 0; i < cookies.length; i++) {
      var parts = cookies[i].split("=");
      var value = parts.slice(1).join("=");
      try {
        var foundKey = decodeURIComponent(parts[0]);
        jar[foundKey] = converter.read(value, foundKey);
        if (key === foundKey) {
          break;
        }
      } catch (e) {
      }
    }
    return key ? jar[key] : jar;
  }
  return Object.create({
    set,
    get,
    remove: function(key, attributes) {
      set(key, "", assign({}, attributes, {
        expires: -1
      }));
    },
    withAttributes: function(attributes) {
      return init(this.converter, assign({}, this.attributes, attributes));
    },
    withConverter: function(converter2) {
      return init(assign({}, this.converter, converter2), this.attributes);
    }
  }, {
    attributes: { value: Object.freeze(defaultAttributes) },
    converter: { value: Object.freeze(converter) }
  });
}
var api = init(defaultConverter, { path: "/" });
class CookieStorageInterface {
  constructor() {
    Object.defineProperty(this, "length", {
      get() {
        return document.cookie ? document.cookie.split("; ").length : 0;
      }
    });
  }
  getItem(name) {
    return api.get(name);
  }
  setItem(name, value) {
    api.set(name, value);
    return true;
  }
  removeItem(name) {
    if (api.get(name)) {
      api.remove(name);
      return true;
    }
    return false;
  }
  key(index) {
    let cookies = document.cookie ? document.cookie.split("; ") : [];
    let keys = [];
    for (let i = 0; i < cookies.length; i++) {
      keys.push(cookies[i].split("=")[0]);
    }
    return keys[index];
  }
}
const CookieStorage = new CookieStorageInterface();
const _global = typeof window !== "undefined" ? window : global || {};
let ls = null;
const VueStorage = {
  install(app, options = {}) {
    const _options = __spreadProps(__spreadValues({}, options), {
      storage: options.storage || "local",
      name: options.name || "$ls"
    });
    if (_options.storage && ["memory", "local", "session", "cookie"].indexOf(_options.storage) === -1) {
      throw new Error(`vue-lsp: Storage "${_options.storage}" is not supported`);
    }
    let store = null;
    switch (_options.storage) {
      case "local":
        store = "localStorage" in _global ? _global.localStorage : null;
        break;
      case "session":
        store = "sessionStorage" in _global ? _global.sessionStorage : null;
        break;
      case "cookie":
        store = CookieStorage;
        break;
      case "memory":
        store = MemoryStorage;
        break;
    }
    if (!store) {
      store = MemoryStorage;
      console.error(`vue-lsp: Storage "${_options.storage}" is not supported your system, use memory storage`);
    }
    ls = new WebStorage(store);
    ls.setOptions(Object.assign(ls.options, {
      namespace: ""
    }, _options || {}));
    if (app.version && app.version > "3") {
      app.config.globalProperties[_options.name] = ls;
      app.config.globalProperties[`$${_options.name}`] = ls;
    } else {
      app[_options.name] = ls;
      Object.defineProperty(app.prototype, `$${_options.name}`, {
        get() {
          return ls;
        }
      });
    }
  }
};
_global.VueStorage = VueStorage;
export { VueStorage as default, ls };
